@@ -1,6 +1,7 @@
 /********************************************
 jmp.c
-copyright 1991, Michael D. Brennan
+copyright 2009-2021,2024, Thomas E. Dickey
+copyright 1991-1993,1995, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,28 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: jmp.c,v 1.4 2009/07/23 23:38:31 tom Exp $
- * @Log: jmp.c,v @
- * Revision 1.4  1995/06/18  19:42:19  mike
- * Remove some redundant declarations and add some prototypes
- *
- * Revision 1.3  1995/04/21  14:20:16  mike
- * move_level variable to fix bug in arglist patching of moved code.
- *
- * Revision 1.2  1993/07/14  13:17:49  mike
- * rm SIZE_T and run thru indent
- *
- * Revision 1.1.1.1  1993/07/03	 18:58:15  mike
- * move source to cvs
- *
- * Revision 5.3	 1993/01/09  19:03:44  mike
- * code_pop checks if the resolve_list needs relocation
- *
- * Revision 5.2	 1993/01/07  02:50:33  mike
- * relative vs absolute code
- *
- * Revision 5.1	 1991/12/05  07:56:10  brennan
- * 1.1 pre-release
+ * $MawkId: jmp.c,v 1.8 2024/07/26 00:34:51 tom Exp $
  */
 
 /* this module deals with back patching jumps, breaks and continues,
@@ -61,7 +41,7 @@ typedef struct jmp {
 static JMP *jmp_top;
 
 void
-code_jmp(int jtype, INST * target)
+code_jmp(int jtype, const INST * target)
 {
     if (error_state)
 	return;
@@ -70,7 +50,7 @@ code_jmp(int jtype, INST * target)
        relocation might make it invalid */
 
     if (target)
-	code2op(jtype, target - (code_ptr + 1));
+	code2op(jtype, (int) (target - (code_ptr + 1)));
     else {
 	register JMP *p = ZMALLOC(JMP);
 
@@ -84,12 +64,12 @@ code_jmp(int jtype, INST * target)
 
 /* patch a jump on the jmp_stack */
 void
-patch_jmp(INST * target)
+patch_jmp(const INST * target)
 {
-    register JMP *p;
-    register INST *source;	/* jmp starts here */
-
     if (!error_state) {
+	register JMP *p;
+	register INST *source;	/* jmp starts here */
+
 #ifdef	DEBUG
 	if (!jmp_top)
 	    bozo("jmp stack underflow");
@@ -98,7 +78,7 @@ patch_jmp(INST * target)
 	p = jmp_top;
 	jmp_top = p->link;
 	source = p->source_offset + code_base;
-	source->op = target - source;
+	source->op = (int) (target - source);
 
 	ZFREE(p);
     }
@@ -121,7 +101,7 @@ BC_new(void)			/* mark the start of a loop */
 }
 
 void
-BC_insert(int type, INST * address)
+BC_insert(int type, const INST * address)
 {
     register BC *p;
 
@@ -136,7 +116,7 @@ BC_insert(int type, INST * address)
     } else {
 	p = ZMALLOC(BC);
 	p->type = type;
-	p->source_offset = address - code_base;
+	p->source_offset = (int) (address - code_base);
 	p->link = bc_top;
 	bc_top = p;
     }
@@ -146,8 +126,7 @@ BC_insert(int type, INST * address)
 void
 BC_clear(INST * B_address, INST * C_address)
 {
-    register BC *p, *q;
-    INST *source;
+    register BC *p;
 
     if (error_state)
 	return;
@@ -155,9 +134,12 @@ BC_clear(INST * B_address, INST * C_address)
     p = bc_top;
     /* pop down to the mark node */
     while (p->type) {
+	INST *source;
+	register BC *q;
+
 	source = code_base + p->source_offset;
-	source->op = (p->type == 'B' ? B_address : C_address)
-	    - source;
+	source->op = (int) ((p->type == 'B' ? B_address : C_address)
+			    - source);
 
 	q = p;
 	p = p->link;
@@ -212,7 +194,7 @@ code_push(INST * code, unsigned len, int scope, FBLOCK * fbp)
 	    p->move_level = code_move_level;
 	    p->fbp = fbp;
 	    if (code)
-		p->offset = code - code_base;
+		p->offset = (int) (code - code_base);
 	    else
 		p->offset = 0;
 	}
@@ -243,7 +225,7 @@ code_pop(INST * target)
     len = p->len;
 
     while (target + len >= code_warn) {
-	target_offset = target - code_base;
+	target_offset = (int) (target - code_base);
 	code_grow();
 	target = code_base + target_offset;
     }
@@ -254,7 +236,7 @@ code_pop(INST * target)
     }
 
     if (p->scope != NO_SCOPE) {
-	target_offset = target - code_base;
+	target_offset = (int) (target - code_base);
 	relocate_resolve_list(p->scope, p->move_level, p->fbp,
 			      p->offset, len, target_offset - p->offset);
     }
