@@ -1,6 +1,6 @@
 /*
 regexp_system.c
-copyright 2009,2010, Thomas E. Dickey
+copyright 2009-2020,2024 Thomas E. Dickey
 copyright 2005, Aleksey Cheusov
 
 This is a source file for mawk, an implementation of
@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
  */
 
 /*
- * $MawkId: regexp_system.c,v 1.35 2010/12/10 17:00:00 tom Exp $
+ * $MawkId: regexp_system.c,v 1.40 2024/08/18 17:20:29 tom Exp $
  */
 #include <sys/types.h>
 #include <stdio.h>
@@ -20,6 +20,8 @@ the GNU General Public License, version 2, 1991.
 #include <stdlib.h>
 #include <errno.h>
 
+#include "mawk.h"
+#include "rexp.h"
 #include "regexp.h"
 
 typedef struct {
@@ -163,7 +165,7 @@ prepare_regexp(char *regexp, const char *source, size_t limit)
 	    case '*':		/* FALLTHRU */
 	    case '.':		/* FALLTHRU */
 	    case '+':		/* FALLTHRU */
-	    case '{':		/* FALLTHRU */
+	    case L_CURL:	/* FALLTHRU */
 	    case '|':		/* FALLTHRU */
 		*tail++ = '\\';
 		*tail++ = ch;
@@ -259,17 +261,29 @@ prepare_regexp(char *regexp, const char *source, size_t limit)
 		}
 		*tail++ = ch;
 		break;
-	    case '{':
-		if (range == 0 &&
-		    ((tail == regexp) ||
-		     (tail[-1] == '*') ||
-		     (tail[-1] == '?'))) {
+	    case L_CURL:
+#ifndef NO_INTERVAL_EXPR
+		if (repetitions_flag) {
+		    *tail++ = ch;
+		    break;
+		} else
+#endif
+		    if (range == 0 &&
+			((tail == regexp) ||
+			 (tail[-1] == '*') ||
+			 (tail[-1] == '?'))) {
 		    *tail++ = '\\';
 		    *tail++ = ch;
 		    break;
 		}
 		/* FALLTHRU */
-	    case '}':
+	    case R_CURL:
+#ifndef NO_INTERVAL_EXPR
+		if (repetitions_flag) {
+		    *tail++ = ch;
+		    break;
+		} else
+#endif
 		if (range == 0)
 		    *tail++ = '\\';
 		*tail++ = ch;
@@ -325,14 +339,17 @@ REcompile(char *regexp, size_t len)
 	    re = NULL;
 	}
     }
+    TRACE(("REcompile(%s) ->%p\n", regexp, re));
     return re;
 }
 
+#ifdef NO_LEAKS
 void
 REdestroy(PTR ptr)
 {
     (void) ptr;
 }
+#endif
 
 /*
  * Test the regular expression in 'q' against the string 'str'.
@@ -344,7 +361,7 @@ REtest(char *str, size_t str_len GCC_UNUSED, PTR q)
 {
     mawk_re_t *re = (mawk_re_t *) q;
 
-    TRACE(("REtest:  \"%s\" ~ /%s/", str, re->regexp));
+    TRACE(("REtest:  \"%s\" ~ /%s/ @%p", str, re->regexp, re));
 
     last_used_regexp = re;
 
@@ -360,16 +377,17 @@ REtest(char *str, size_t str_len GCC_UNUSED, PTR q)
 #define MAX_MATCHES 100
 
 char *
-REmatch(char *str, size_t str_len GCC_UNUSED, PTR q, size_t *lenp)
+REmatch(char *str, size_t str_len GCC_UNUSED, PTR q, size_t *lenp, int no_bol)
 {
     mawk_re_t *re = (mawk_re_t *) q;
     regmatch_t match[MAX_MATCHES];
+    int flags = (no_bol ? REG_NOTBOL : 0);
 
-    TRACE(("REmatch:  \"%s\" ~ /%s/", str, re->regexp));
+    TRACE(("REmatch: %s \"%s\" ~ /%s/", no_bol ? "any" : "1st", str, re->regexp));
 
     last_used_regexp = re;
 
-    if (!regexec(&re->re, str, (size_t) MAX_MATCHES, match, 0)) {
+    if (!regexec(&re->re, str, (size_t) MAX_MATCHES, match, flags)) {
 	*lenp = (size_t) (match[0].rm_eo - match[0].rm_so);
 	TRACE(("=%i/%lu\n", match[0].rm_so, (unsigned long) *lenp));
 	return str + match[0].rm_so;
@@ -379,6 +397,7 @@ REmatch(char *str, size_t str_len GCC_UNUSED, PTR q, size_t *lenp)
     }
 }
 
+#if defined(DEBUG)
 void
 REmprint(void *m, FILE *f)
 {
@@ -387,6 +406,7 @@ REmprint(void *m, FILE *f)
     /* no debugging code available */
     abort();
 }
+#endif
 
 static char error_buffer[2048];
 
