@@ -1,6 +1,6 @@
 /********************************************
 repl.h
-copyright 2009-2014,2020, Thomas E. Dickey
+copyright 2009-2023,2024, Thomas E. Dickey
 copyright 1991,1993, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: repl.h,v 1.9 2020/07/17 23:16:52 tom Exp $
+ * $MawkId: repl.h,v 1.12 2024/08/18 09:55:30 tom Exp $
  */
 
 /* repl.h */
@@ -27,6 +27,12 @@ typedef struct re_data {
     int is_empty;		/* check if pattern is empty */
 } RE_DATA;
 
+typedef struct re_node {
+    RE_DATA re;			/* keep this first, for re_destroy() */
+    STRING *sval;
+    struct re_node *link;
+} RE_NODE;
+
 /*
  * re_compile returns a RE_DATA*, but mawk handles it as a PTR thereafter.
  */
@@ -36,10 +42,10 @@ typedef struct re_data {
 #define refRE_DATA(re)  ((PTR) &(re))
 
 PTR re_compile(STRING *);
-char *re_uncompile(PTR);
+STRING *re_uncompile(PTR);
 
 CELL *repl_compile(STRING *);
-char *repl_uncompile(CELL *);
+const STRING *repl_uncompile(CELL *);
 void re_destroy(PTR);
 void repl_destroy(CELL *);
 CELL *replv_cpy(CELL *, CELL *);
