@@ -1,6 +1,6 @@
 /********************************************
 rexp.c
-copyright 2008-2017,2020, Thomas E. Dickey
+copyright 2008-2023,2024, Thomas E. Dickey
 copyright 1991-1993,1996, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: rexp.c,v 1.34 2020/10/23 08:07:50 tom Exp $
+ * $MawkId: rexp.c,v 1.45 2024/08/05 20:59:33 tom Exp $
  */
 
 /*  op precedence  parser for regular expressions  */
@@ -65,6 +65,8 @@ static  short  table[10][10]  =  {
 
 #define	 STACKSZ   64
 
+static const char *REs_type(STATE * p);
+
 static jmp_buf err_buf;		/*  used to trap on error */
 
 #if OPT_TRACE > 0
@@ -115,6 +117,9 @@ typedef struct {
     int prec;
 } OPS;
 
+/* duplicate_m() relies upon copying machines whose size is 1, i.e., atoms */
+#define BigMachine(mp) (((mp)->stop - (mp)->start) > 1)
+
 STATE *
 REcompile(char *re, size_t len)
 {
@@ -170,13 +175,22 @@ REcompile(char *re, size_t len)
 	     *   convert m{3,} to mmm* (with a limit of MAX_INT)
 	     *   convert m{3,10} to mmm* with a limit of 10
 	     */
+	    TRACE(("interval {%ld,%ld}\n", (long) intrvalmin, (long) intrvalmax));
 	    if (intrvalmin == 0) {	/* zero or more */
 		switch (intrvalmax) {
 		case 0:
 		    /* user stupidity: m{0} or m{0,0} 
 		     * don't add this re token
 		     */
-		    if (op_ptr != op_stack) {
+		    if (m_ptr < m_stack) {
+			t = RE_lex(++m_ptr);
+			if (t != T_NONE) {
+			    continue;
+			} else {
+			    m_stack[0] = RE_any();	/* FIXME: RE_none? */
+			    m_ptr = m_stack;
+			}
+		    } else if (op_ptr != op_stack) {
 			/* no previous re */
 			RE_free(m_ptr->start);
 			m_ptr--;
@@ -187,8 +201,6 @@ REcompile(char *re, size_t len)
 				if (op_ptr->token == T_LP) {
 				    if (op_ptr == op_stack) {
 					op_ptr->token = T_NONE;
-				    } else {
-					--op_ptr;
 				    }
 				    break;
 				}
@@ -220,6 +232,8 @@ REcompile(char *re, size_t len)
 		    RE_close_limit(m_ptr, intrvalmin, intrvalmax);
 		    TRACE(("RE_lex token %s\n", token_name(T_Q)));
 		}
+	    } else if (BigMachine(m_ptr)) {
+		RE_poscl_limit(m_ptr, intrvalmin, intrvalmax);
 #ifdef NO_RI_LOOP_UNROLL
 	    } else if (intrvalmin >= 1) {	/* one or more */
 		RE_poscl_limit(m_ptr, intrvalmin, intrvalmax);
@@ -230,12 +244,12 @@ REcompile(char *re, size_t len)
 		TRACE(("RE_lex token %s\n", token_name(T_PLUS)));
 #endif
 	    } else {		/* n or more */
-		register int i;
+		register Int i;
 		/* copy 2 copies of m_ptr, use 2nd copy to replace
 		   the first copy that gets swallowed by concat */
 		MACHINE *result_mp = m_ptr;
-		MACHINE *concat_mp = (MACHINE *) (m_ptr + 1);
-		MACHINE *new_mp = (MACHINE *) (m_ptr + 2);
+		MACHINE *concat_mp = (m_ptr + 1);
+		MACHINE *new_mp = (m_ptr + 2);
 		duplicate_m(concat_mp, result_mp);
 		duplicate_m(new_mp, result_mp);
 		for (i = 2; i <= intrvalmin; i++) {
@@ -248,7 +262,7 @@ REcompile(char *re, size_t len)
 	    break;
 #endif /* ! NO_INTERVAL_EXPR */
 
-	case 0:		/*  end of reg expr   */
+	case T_NONE:		/*  end of reg expr   */
 	    if (op_ptr->token == 0) {
 		/*  done   */
 		if (m_ptr == m_stack) {
@@ -341,6 +355,7 @@ REcompile(char *re, size_t len)
     }
 }
 
+#ifdef NO_LEAKS
 void
 REdestroy(STATE * ptr)
 {
@@ -348,9 +363,9 @@ REdestroy(STATE * ptr)
     int n = 0;
     STATE *q = ptr;
 
-    TRACE(("REdestroy %p\n", ptr));
+    TRACE(("REdestroy %p\n", (void *) ptr));
     while (!done) {
-	TRACE(("...%d type %d\n", n, q->s_type));
+	TRACE(("...destroy[%d] %p type %s\n", n, q, REs_type(q)));
 	switch (q->s_type) {
 	case M_ACCEPT:
 	    done = 1;
@@ -368,6 +383,7 @@ REdestroy(STATE * ptr)
     }
     RE_free(ptr);
 }
+#endif /* NO_LEAKS */
 
 /* getting here means a logic flaw or unforeseen case */
 void
@@ -411,7 +427,7 @@ duplicate_m(MACHINE * newmp, MACHINE * oldmp)
 {
     register STATE *p;
     p = (STATE *) RE_malloc(2 * STATESZ);
-    memcpy(p, oldmp->start, 2 * STATESZ);
+    RE_copy_states(p, oldmp->start, 2);
     newmp->start = (STATE *) p;
     newmp->stop = (STATE *) (p + 1);
 }
